from .cmd import CMD

class BOOT:
  BOOT_CMD_READ_BOOT_VERSION =      0x00
  BOOT_CMD_READ_BOOT_NAME    =      0x01
  BOOT_CMD_READ_FIRM_VERSION =      0x02
  BOOT_CMD_READ_FIRM_NAME    =      0x03
  BOOT_CMD_FLASH_ERASE       =      0x04
  BOOT_CMD_FLASH_WRITE       =      0x05
  BOOT_CMD_FLASH_READ        =      0x06
  BOOT_CMD_JUMP_TO_FW        =      0x08

  BOOT_CMD_TAG_ERASE         =      0x10
  BOOT_CMD_TAG_WRITE         =      0x11
  BOOT_CMD_TAG_READ          =      0x12
  BOOT_CMD_TAG_VERIFY        =      0x13


  BOOT_CMD_UPDATE_ERASE      =      0x20
  BOOT_CMD_UPDATE_WRITE      =      0x21
  BOOT_CMD_UPDATE_READ       =      0x22
  BOOT_CMD_UPDATE_FW         =      0x24

  
  def __init__(self, cmd):
    self.cmd = cmd
  
  def bootProcessCmd(self, command):
    match command:
      case "1":
        self.bootReadBootVersion()

      case "2":
        self.bootReadBootName()

      case "3":
        self.bootFlashRead()
      
      case self.BOOT_CMD_READ_FIRM_NAME:
        pass

      case "7":
        self.bootFlashWrite()

      case "4":
        self.bootFlashErase()

      case self.BOOT_CMD_FLASH_READ:
        pass

      case self.BOOT_CMD_JUMP_TO_FW:
        pass

      case self.BOOT_CMD_TAG_READ:
        pass

      case self.BOOT_CMD_TAG_WRITE:
        pass

      case self.BOOT_CMD_TAG_VERIFY:
        pass

      case self.BOOT_CMD_TAG_ERASE:
        pass

      case self.BOOT_CMD_UPDATE_ERASE:
        pass

      case self.BOOT_CMD_UPDATE_WRITE:
        pass

      case self.BOOT_CMD_UPDATE_READ:
        pass
      
      case self.BOOT_CMD_UPDATE_FW:
        pass
      
      
      case _:
        print("Input invalid")

  def bootReadBootVersion(self):
    if(self.cmd.cmdSendCmdRxResp(self.BOOT_CMD_READ_BOOT_VERSION, [], 0, 1) == True):
          version = ""
          for char in self.cmd.rx_packet.buffer:
            version+= chr(char)
          print(f"BootLoader Version: {version}")
    else:
      print(f"Read BootLoader Version Fail!")
      
  def bootReadBootName(self):
    if(self.cmd.cmdSendCmdRxResp(self.BOOT_CMD_READ_BOOT_NAME, [], 0, 1) == True):
          name = ""
          for char in self.cmd.rx_packet.buffer:
            name+= chr(char)
          print(f"BootLoader Name: {name}")
    else:
      print(self.cmd.rx_packet.buffer)
      print(f"Read BootLoader Name Fail!")
      
  def bootFlashRead(self):
    addr = input("Input Address (ex: 0x8000000): ")
    length = input("Input Length (ex: 1024)      : ")
    addr = int(addr, 16)
    length = int(length)
    data = [addr >> 0 & 0xFF, addr >> 8 & 0xFF,addr >> 16 & 0xFF,addr >> 24 & 0xFF, length >> 0 & 0xFF, length >> 8 & 0xFF, length >> 16 & 0xFF, length >> 24 & 0xFF]
    if(self.cmd.cmdSendCmdRxResp(self.BOOT_CMD_FLASH_READ, data, len(data), 3) == True):
      print(f"Flash Data:")
      data = []
      for i in range(0, len(self.cmd.rx_packet.buffer), 4):
        if((i+4) > len(self.cmd.rx_packet.buffer)):
          data.append(self.cmd.rx_packet.buffer[i])
          for j in range(1, (self.cmd.rx_packet.buffer)%4, 1):
            data[-1] |= self.cmd.rx_packet.buffer[i] << 8*i
        else:
          data.append((self.cmd.rx_packet.buffer[i]) << 0)
          data[-1] |= (self.cmd.rx_packet.buffer[i+1]) << 8
          data[-1] |= (self.cmd.rx_packet.buffer[i+2]) << 16
          data[-1] |= (self.cmd.rx_packet.buffer[i+3]) << 24
          
      for i in range(0, len(data), 4):
        if((i+4) > len(data)):
          for j in range(len(data)%4):
            print("{0}: {1}\r\n".format(hex((addr + 32*i)), data[j]))
        else:
          print("{0}: {1} | {2} | {3} | {4}\r\n".format(hex((addr + 32*i)), hex(data[i]), hex(data[i+1]), hex(data[i+2]), hex(data[i+3])))  
      
    else:
      print(f"Read Flash Fail!")
      
  def bootFlashWrite(self, addr, data, length):
    pass
  
  def bootFlashErase(self):
    addr = input("Input Address (ex: 0x8000000): ")
    length = input("Input Length (ex: 1024)      : ")
    addr = int(addr, 16)
    length = int(length)
    data = [addr >> 0 & 0xFF, addr >> 8 & 0xFF,addr >> 16 & 0xFF,addr >> 24 & 0xFF, length >> 0 & 0xFF, length >> 8 & 0xFF, length >> 16 & 0xFF, length >> 24 & 0xFF]
    if(self.cmd.cmdSendCmdRxResp(self.BOOT_CMD_FLASH_ERASE, data, len(data), 3) == True):
      print("Erase Done!")
    else:
      print("Erase Erorr!")


from .cmd import CMD
from .boot import BOOT
import os
import colorama
import click

@click.command()
@click.option("--port", prompt = "Enter port", help="The name port")
@click.option("--baud", prompt = "Enter Baud rate", help="The Baud rate")
def main(port, baud):
    cmd = CMD(port, baud)
    cmd.init()
    cmd.open()
    while True:
      print(colorama.Fore.GREEN + "-------------------------------------")
      print("1. Led")
      print("2. BootLoader")
      print("3. Exit")
      print("-------------------------------------")
      option = option = input("Choose type:  ")
      
      match option:
        case "1":
          os.system('clear')
          led(cmd)
          
        case "2":
          os.system('clear')
          boot(cmd)
        
        case "3":
          print("Good bye!")
          break
          
        case _:
          print(colorama.Fore.RED + "Input invalid!")
          _ = input("Press anykey....")  
        
      os.system('clear')


def led(cmd):
  while True:
    print(colorama.Fore.GREEN + "-------------------LED TEST------------------")
    print("1. Led ON")
    print("2. Led OFF")
    print("3. Led Toggle")
    print("4. Exit")
    print("---------------------------------------------")
    option = input("Choose type:   ")
    if(option == "1"):
      if(cmd.cmdSendCmdRxResp(0x10, [1], 1, 1) == True):
        print(colorama.Fore.GREEN + "Led ON success!")
      else:
        print(colorama.Fore.RED + "Led ON Fail!")
    elif(option == "2"):
      if(cmd.cmdSendCmdRxResp(0x10, [2], 1, 1) == True):
        print(colorama.Fore.GREEN +  "Led OFF success!")
      else:
        print(colorama.Fore.RED + "Led OFF Fail!")
    elif(option == "3"):
      if(cmd.cmdSendCmdRxResp(0x10, [3], 1, 1) == True):
        print(colorama.Fore.GREEN +  "Led Toggle success!")
      else:
        print(colorama.Fore.RED + "Led Toggle Fail!")
    elif(option == "4"):
      os.system('clear')
      break
    else:
      print(colorama.Fore.RED + "Input invalid!")
    _ = input("Press anykey....")
    os.system('clear')
      

def boot(cmd):
  bootMCU = BOOT(cmd)
  while True:
    print(colorama.Fore.GREEN + "------------------- BOOT ------------------")
    print("1. Read BootLoader Version")
    print("2. Read BootLoader Name")
    print("3. Read Flash")
    print("4. Erase Flash")
    print("5. Flash New Firmware")
    print("6. Test New Firmware")
    print("7. Update New Firmware")
    print("8. Exit")
    print("-------------------------------------------")
    
    option = input("Choose type:   ")
    
    bootMCU.bootProcessCmd(option)
    
    if(option == "8"):
      break
    _ = input("Press anykey....")
    os.system('clear')
  
  
if __name__ == "__main__":
  main()


import colorama
from .uart import UART
from .bsp import delay, millis


class CmdPacket:
  cmd = 0
  dir = 0
  error = 0
  length = 0
  check_sum = 0
  check_sum_recv = 0
  buffer = []
  data= 0

class CMD:
  
  CMD_OK  =                 0
  
  CMD_DIR_M_TO_S  =         0
  CMD_DIR_S_TO_M  =         1
  
  CMD_STX =                 0x02
  CMD_EXT =                 0x03

  CMD_STATE_WAIT_STX=       0
  CMD_STATE_WAIT_CMD=       1
  CMD_STATE_WAIT_DIR=       2
  CMD_STATE_WAIT_ERROR=     3
  CMD_STATE_WAIT_LENGTH_L=  4
  CMD_STATE_WAIT_LENGTH_H=  5
  CMD_STATE_WAIT_DATA=      6
  CMD_STATE_WAIT_CHECKSUM=  7
  CMD_STATE_WAIT_ETX=       8

  CMD_MAX_DATA_LENGTH=      256
  
  is_init = False
  state = CMD_STATE_WAIT_STX
  pre_time = 0
  index = 0
  error = 0
  rx_packet = CmdPacket()
  tx_packet = CmdPacket()
  
  def __init__(self, ch, baud):
    self.uart = UART(ch, baud)
    
  def init(self):
    self.is_init = True
    self.state = self.CMD_STATE_WAIT_STX

  def open(self):
    self.pre_time = millis()
    return self.uart.open()
  
  def close(self):
    return self.uart.close()
  
  def cmdReceivePacket(self):
    ret = False
    rx_data = 0
    if(self.uart.available()):
      rx_data = self.uart.read()
    else:
      return False
    if(millis() - self.pre_time >= 100):
      self.state = self.CMD_STATE_WAIT_STX
      return False
    
    self.pre_time = millis()
    
    rx_data = int.from_bytes(rx_data, "little")
    match self.state:
      case  self.CMD_STATE_WAIT_STX:
        if(rx_data == self.CMD_STX):
          self.state = self.CMD_STATE_WAIT_CMD
          self.rx_packet.check_sum = 0
      
      case self.CMD_STATE_WAIT_CMD :
        self.rx_packet.cmd = rx_data
        self.rx_packet.check_sum ^= rx_data
        self.state = self.CMD_STATE_WAIT_DIR
        
      case self.CMD_STATE_WAIT_DIR:
        self.rx_packet.dir = rx_data
        self.rx_packet.check_sum ^= rx_data
        self.state = self.CMD_STATE_WAIT_ERROR
        
      case self.CMD_STATE_WAIT_ERROR:
        self.rx_packet.error = rx_data
        self.rx_packet.check_sum ^= rx_data
        self.rx_packet.length = 0
        self.rx_packet.buffer = []
        self.index = 0
        self.state = self.CMD_STATE_WAIT_LENGTH_L
        
      case self.CMD_STATE_WAIT_LENGTH_L:
        self.rx_packet.length = rx_data
        self.rx_packet.check_sum ^= rx_data
        self.state = self.CMD_STATE_WAIT_LENGTH_H
        
      case self.CMD_STATE_WAIT_LENGTH_H:
        self.rx_packet.length |= rx_data << 8
        self.rx_packet.check_sum ^= rx_data
        if(self.rx_packet.length > 0):
          self.rx_packet.buffer = []
          self.state = self.CMD_STATE_WAIT_DATA
        else:
          self.state = self.CMD_STATE_WAIT_CHECKSUM
      
      case self.CMD_STATE_WAIT_DATA:
        self.rx_packet.buffer.append(rx_data)
        self.rx_packet.check_sum ^= rx_data
        self.index += 1
        if(self.index == self.rx_packet.length):
          self.state = self.CMD_STATE_WAIT_CHECKSUM
        else:
          self.state = self.CMD_STATE_WAIT_DATA
          
      case self.CMD_STATE_WAIT_CHECKSUM:
        self.rx_packet.check_sum_recv = rx_data
        self.state = self.CMD_STATE_WAIT_ETX
      
      case self.CMD_STATE_WAIT_ETX:
        if(rx_data == self.CMD_EXT):
          if(self.rx_packet.check_sum == self.rx_packet.check_sum_recv):
            ret = True
        self.state = self.CMD_STATE_WAIT_STX  
    return ret
        
        
  def cmdSendCmd(self, cmd, data, length):
    self.tx_packet.buffer = []
    self.tx_packet.buffer.append(self.CMD_STX)
    self.tx_packet.buffer.append(cmd)
    self.tx_packet.buffer.append(self.CMD_DIR_M_TO_S)
    self.tx_packet.buffer.append(self.CMD_OK)
    self.tx_packet.buffer.append((length >> 0) & 0xFF)
    self.tx_packet.buffer.append((length >> 8) & 0xFF)
    
    for i in range(length):
      self.tx_packet.buffer.append(data[i])
    
    checksum = 0
    
    for i in range(1, length + 6, 1):
      checksum ^= self.tx_packet.buffer[i]
      
    self.tx_packet.buffer.append(checksum)
    self.tx_packet.buffer.append(self.CMD_EXT)
    self.uart.write(self.tx_packet.buffer)
    
  
  def cmdSendResp(self, cmd, err_code, data, length):
    self.tx_packet.buffer.append(self.CMD_STX)
    self.tx_packet.buffer.append(cmd)
    self.tx_packet.buffer.append(self.CMD_DIR_M_TO_S)
    self.tx_packet.buffer.append(err_code)
    self.tx_packet.buffer.append((length >> 0) & 0xFF)
    self.tx_packet.buffer.append((length >> 8) & 0xFF)
    
    for i in range(length):
      self.tx_packet.buffer.append(data[i])
    
    checksum = 0
    
    for i in range(length + 5):
      i+=1
      checksum ^= self.tx_packet.buffer[i]
      
    self.tx_packet.buffer.append(checksum)
    self.tx_packet.buffer.append(self.CMD_EXT)
    
    self.uart.write(self.tx_packet.buffer)
  
  def cmdSendCmdRxResp(self, cmd, data, length, timout):
    ret = False
    pretime = 0
    
    self.cmdSendCmd(cmd, data, length)
    
    pretime = millis()

    while True:
      if(self.cmdReceivePacket() == True):
        ret = True
        break
      
      if(millis() - pretime > timout):
        break
    
    return ret
    
  